
class LissaPath {
  FPoint pos;
  FPoint vel;
  FPoint force;
  FPoint center;
  FPoint radii;
  FPoint speeds;

  FPoint eyeDisp;
  float eyeTargetAng;
  int    who;
  int    id;

  float DAMPING             = random(0.93, 0.970);
  float FORCE_MYCENTER_ATTR = random(0.05, 0.12);
  float FORCE_OTHER_SEP     = random(25, 35);
  float FORCE_CURSOR        = 10;
  float FORCE_BABY          = 1200;

  LissaPath (float x, float y, int w, int i){

    id = i;
    who = w;
    center = new FPoint(x,y);

    float rx = random(4,10);
    float ry = random(4,10);
    radii  = new FPoint(rx,ry);

    float sx = random(1, 4);
    float sy = random(1, 4);
    speeds = new FPoint(sx,sy);

    pos = new FPoint(x,y);
    vel = new FPoint(0,0);
    force = new FPoint(0,0);

    eyeDisp = new FPoint(0,0);
    eyeTargetAng = 0;
  }

  void update(){

      if (id == 21){
        float dmx = (width- (0.75*(mouseX-width/2) + width/2)) - pos.x;
        float dmy = height*0.85 /*min(mouseY, height*0.95)*/ - (pos.y +20);

        force.x = (dmx * 0.015);
        force.y = (dmy * 0.085);
        force.y += 0.75*sin(millis()/135.0);
        return;
      }
      
    force.x = 0;
    force.y = 0;
    float dx,dy,dh;
    float dir = ((id % 2) == 0) ? -1 : 1;

    float t = millis()/1000.0;
    float cx = center.x + radii.x*sin(speeds.x * t);
    float cy = center.y + radii.y*sin(speeds.y * t);


    // restorative constant force around personal centerpoint
    float dcx = cx - pos.x;
    float dcy = cy - pos.y;
    float dch = sqrt(dcx*dcx + dcy*dcy);
    if (dch > 1.0){
      //dcx /= dch;
      //dcy /= dch;
      float fcx = dcx * FORCE_MYCENTER_ATTR; 
      float fcy = dcy * FORCE_MYCENTER_ATTR;
      force.x += fcx;
      force.y += fcy;
    } 

    // inverse repulsion force away from nearby who-mates
    for (int i=0; i<nLissaPaths; i++){
      LissaPath L = lissaPathArray[i];
      if ((L.id != id) && (L.who == who)){
        dx = pos.x - L.pos.x;
        dy = pos.y - L.pos.y;
        dh = (dx*dx + dy*dy);
        if (dh > 0.001){
          dx /= dh;
          dy /= dh;
          float fwx = dx * FORCE_OTHER_SEP;
          float fwy = dy * FORCE_OTHER_SEP;
          force.x += fwx - dir*dy*0.4;
          force.y += fwy + dir*dx*0.4;
        } 

      } 
      else if (L.who == 2){ // baby
        dx = pos.x - L.pos.x;
        dy = pos.y - L.pos.y;
        dh = sqrt(dx*dx + dy*dy);
        if (dh > 8){
          dx /= pow(dh,2.5);
          dy /= pow(dh,2.5);
          float fwx = dx * FORCE_BABY;
          float fwy = dy * FORCE_BABY;
          force.x -= fwx;
          force.y -= fwy;
        } 
      } 

    }



  }

  //------------------------------------
  void integrate(){
    vel.x += force.x;
    vel.y += force.y;
    vel.x *= DAMPING;
    vel.y *= DAMPING;
    pos.x += vel.x;
    pos.y += vel.y;
  }

  //------------------------------------
  FPoint getCurrentPoint(){
    return pos;
  }

  //------------------------------------
  void render(){




    float vx = vel.x;
    float vy = vel.y;
    float vh = sqrt(vx*vx + vy*vy);
    float ang = atan2(vy,vx);
    vh *= 2.5;
    vh = min(vh,10); 

    float alp = 10 + 0.25*sq(vh);
    noStroke();
    switch(who){
    case 0: 
      fill(100,0,100, alp); 
      break;
    case 1: 
      fill(255,0,0, alp);   
      break;
    case 2:
      fill(0,170,240, alp);
      break;
    case 3:
      fill(blobColor);
      break;
    }

    if ((id%9 == 1) || (id == 21)) { // eye
      pushMatrix();
      translate(pos.x, pos.y, 0);
      if (id != 21){
        scale(1.25,1.25,1);
      } else {
        scale(0.9,0.9, 1);
      }

      fill(20,0,0, 20);
      ellipse(0,0, 19,19);
      translate(0,0,0.02);
      ellipse(0,0, 17,17);
      translate(0,0,0.02);
      fill(255);
      ellipse(0,0, 13,13);

      // pupil
      float targetX = mouseX;
      float targetY = mouseY;
      float dmx = mouseX - blurMx;
      float dmy = mouseY - blurMy;
      float dmh = sqrt(dmx*dmx + dmy*dmy);
      if (id == 21){ // for pet
        ;
        
      } else if (id != 19){ // for parents
        if (dmh < 60.0){
          targetX = lissaPathArray[19].pos.x;
          targetY = lissaPathArray[19].pos.y;
        }
      
      } else { // for baby

        if (dmh > 90.0){ // look straight ahead
          targetX = lissaPathArray[19].pos.x;
          targetY = lissaPathArray[19].pos.y;
        }

        float n = noise(millis()/1000.0);
        if (n > 0.6){
          float m = noise(millis()/888.0);
          if (m < 0.5){
            targetX = lissaPathArray[1].pos.x;
            targetY = lissaPathArray[1].pos.y;
          } 
          else {
            targetX = lissaPathArray[10].pos.x;
            targetY = lissaPathArray[10].pos.y;
          }
        }
      }
      
      if (random(1.0) < 0.005){
        
        translate(0,0, 0.03);
        fill(100,40,40, 150);
        rect(-6.5,-1, 13,2);
       
        
      } else {


      float edx = targetX - pos.x;
      float edy = targetY - pos.y;
      float edh = sqrt(edx*edx + edy*edy);
      if (edh > 0.001){

        float A = 0.60;
        float B = 1.0 - A;
        float angl = atan2(edy,edx);
        eyeTargetAng = A*eyeTargetAng + B*angl;

        edx = cos(eyeTargetAng);
        edy = sin(eyeTargetAng);
        edx *= 2.0;
        edy *= 2.0;
        eyeDisp.x = A*eyeDisp.x + B*edx;
        eyeDisp.y = A*eyeDisp.y + B*edy;
        translate(eyeDisp.x, eyeDisp.y, 0);
      }
      translate(0,0,0.02);
      fill(30,54,84);
      ellipse(0,0, 6+vh/6,6+vh/6);
      translate(0,0,0.02);
      fill(255,233,180);
      ellipse(1,-1, 2,2);
      }
      
      
      popMatrix();


    } 
    else { //regular dot

      pushMatrix();
      translate(pos.x, pos.y); 
      rotate(ang); 
      ellipse(0,0, 3+1.5*vh, 3+vh); 
      popMatrix();

    }
  }

}


